import factory

from contact import signals
from contact.models import Contact


@factory.django.mute_signals(signals.post_save)
class ContactFactory(factory.django.DjangoModelFactory):
    name = factory.Faker("name")
    email = factory.Faker("email")
    message = factory.Faker("paragraph")

    class Meta:
        model = Contact
        django_get_or_create = ("email",)



from decimal import Decimal

from django.contrib.auth import get_user_model
from django.urls import reverse
from datetime import timedelta
from django.utils import timezone
from rest_framework.test import APITestCase
from product.factories.product import ProductFactory
from product.models.favourite import ProductFavourite
from user.factories.account import UserAccountFactory

User = get_user_model()


class ProductFavouriteFilterTest(APITestCase):
    def setUp(self):
        ProductFavourite.objects.all().delete()

        self.user1 = UserAccountFactory(first_name="John", last_name="Doe")
        self.user2 = UserAccountFactory(first_name="Jane", last_name="Smith")
        self.product1 = ProductFactory(price=Decimal("100.00"), active=True)
        self.product2 = ProductFactory(price=Decimal("50.00"), active=True)

        self.now = timezone.now()

        self.old_favourite = ProductFavourite.objects.create(
            user=self.user1,
            product=self.product1,
        )
        ProductFavourite.objects.filter(id=self.old_favourite.id).update(
            created_at=self.now - timedelta(days=60),
            updated_at=self.now - timedelta(days=50),
        )
        self.old_favourite.refresh_from_db()

        self.recent_favourite = ProductFavourite.objects.create(
            user=self.user1,
            product=self.product2,
        )
        ProductFavourite.objects.filter(id=self.recent_favourite.id).update(
            created_at=self.now - timedelta(hours=2),
            updated_at=self.now - timedelta(hours=1),
        )
        self.recent_favourite.refresh_from_db()

        self.user2_favourite = ProductFavourite.objects.create(
            user=self.user2,
            product=self.product1,
        )
        ProductFavourite.objects.filter(id=self.user2_favourite.id).update(
            created_at=self.now - timedelta(hours=3),
            updated_at=self.now - timedelta(hours=2),
        )
        self.user2_favourite.refresh_from_db()

        self.high_sort_favourite = ProductFavourite.objects.create(
            user=self.user2,
            product=self.product2,
        )
        ProductFavourite.objects.filter(id=self.high_sort_favourite.id).update(
            created_at=self.now - timedelta(hours=4),
            updated_at=self.now - timedelta(hours=3),
        )
        self.high_sort_favourite.refresh_from_db()

    def test_timestamp_filters(self):
        url = reverse("product-favourite-list")

        created_after_date = self.now - timedelta(days=30)
        response = self.client.get(
            url,
            {"created_after": created_after_date.isoformat()},
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        self.assertNotIn(self.old_favourite.id, result_ids)

        created_before_date = self.now - timedelta(days=30)
        response = self.client.get(
            url,
            {"created_before": created_before_date.isoformat()},
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        if self.old_favourite.created_at < created_before_date:
            self.assertIn(self.old_favourite.id, result_ids)

    def test_id_filters(self):
        url = reverse("product-favourite-list")

        response = self.client.get(url, {"id": self.recent_favourite.id})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(
            response.data["results"][0]["id"], self.recent_favourite.id
        )

        response = self.client.get(url, {"product_id": self.product1.id})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.old_favourite.id, result_ids)
        self.assertIn(self.user2_favourite.id, result_ids)

        response = self.client.get(url, {"user_id": self.user1.id})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.old_favourite.id, result_ids)
        self.assertIn(self.recent_favourite.id, result_ids)

    def test_uuid_filter(self):
        url = reverse("product-favourite-list")

        response = self.client.get(
            url, {"uuid": str(self.recent_favourite.uuid)}
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(
            response.data["results"][0]["id"], self.recent_favourite.id
        )

    def test_sort_order_filters_if_available(self):
        url = reverse("product-favourite-list")

        has_sort_order = hasattr(ProductFavourite, "sort_order")

        if not has_sort_order:
            return

        response = self.client.get(url, {"sort_order": 200})
        self.assertEqual(response.status_code, 200)

    def test_camel_case_filters(self):
        url = reverse("product-favourite-list")

        created_after_date = self.now - timedelta(days=30)

        response = self.client.get(
            url,
            {
                "createdAfter": created_after_date.isoformat(),
                "userId": self.user1.id,
            },
        )
        self.assertEqual(response.status_code, 200)

    def test_existing_filters_still_work(self):
        url = reverse("product-favourite-list")

        response = self.client.get(
            url,
            {
                "user_id": self.user2.id,
                "product_id": self.product1.id,
            },
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 1)
        self.assertIn(self.user2_favourite.id, result_ids)

    def test_complex_filter_combinations(self):
        url = reverse("product-favourite-list")

        response = self.client.get(
            url,
            {
                "user_id": self.user1.id,
                "ordering": "-created_at",
            },
        )

        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]

        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.old_favourite.id, result_ids)
        self.assertIn(self.recent_favourite.id, result_ids)

    def test_filter_with_ordering(self):
        url = reverse("product-favourite-list")

        response = self.client.get(
            url, {"user_id": self.user1.id, "ordering": "-created_at"}
        )
        self.assertEqual(response.status_code, 200)

        results = response.data["results"]
        self.assertEqual(len(results), 2)

        favourite_ids = [r["id"] for r in results]
        self.assertIn(self.old_favourite.id, favourite_ids)
        self.assertIn(self.recent_favourite.id, favourite_ids)

    def test_multiple_products_filter(self):
        url = reverse("product-favourite-list")

        response = self.client.get(
            url,
            {
                "product_id": self.product1.id,
                "ordering": "created_at",
            },
        )
        self.assertEqual(response.status_code, 200)

        results = response.data["results"]
        self.assertEqual(len(results), 2)

        result_ids = [r["id"] for r in results]
        self.assertIn(self.old_favourite.id, result_ids)
        self.assertIn(self.user2_favourite.id, result_ids)

    def test_timestamp_range_filters(self):
        url = reverse("product-favourite-list")

        created_after = self.now - timedelta(days=7)

        response = self.client.get(
            url,
            {
                "created_after": created_after.isoformat(),
            },
        )
        self.assertEqual(response.status_code, 200)

        result_ids = [r["id"] for r in response.data["results"]]

        self.assertNotIn(self.old_favourite.id, result_ids)

    def tearDown(self):
        ProductFavourite.objects.all().delete()



import factory
from django.contrib.auth import get_user_model
from django.contrib.auth.hashers import make_password

from user.factories.address import UserAddressFactory

User = get_user_model()


class UserAccountFactory(factory.django.DjangoModelFactory):
    email = factory.Faker("email")
    plain_password = factory.Faker("password")
    password = factory.LazyAttribute(
        lambda o: make_password(
            o.plain_password if o.plain_password else "password"
        )
    )
    first_name = factory.Faker("first_name")
    last_name = factory.Faker("last_name")
    phone = factory.Faker("phone_number")
    city = factory.Faker("city")
    zipcode = factory.Faker("postcode")
    address = factory.Faker("address")
    place = factory.Faker("address")
    image = factory.django.ImageField(
        filename="user_image.jpg", width=800, height=800, color="blue"
    )
    birth_date = factory.Faker("date_of_birth")
    twitter = factory.Faker("url")
    linkedin = factory.Faker("url")
    facebook = factory.Faker("url")
    instagram = factory.Faker("url")
    website = factory.Faker("url")
    youtube = factory.Faker("url")
    github = factory.Faker("url")
    bio = factory.Faker("paragraph")
    is_staff = False
    is_superuser = False

    class Meta:
        model = User
        django_get_or_create = ("email",)
        skip_postgeneration_save = True
        exclude = ("plain_password",)

    class Params:
        admin = factory.Trait(is_superuser=True, is_staff=True)
        staff = factory.Trait(is_staff=True)

    @factory.post_generation
    def num_addresses(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            UserAddressFactory.create_batch(extracted, user=self)



# Generated by Django 5.0.3 on 2024-03-09 20:14
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        (
            "product",
            "0004_product_search_document_product_search_index_dirty_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="product",
            name="search_document_dirty",
            field=models.BooleanField(db_index=True, default=False),
        ),
    ]



import json
import logging
import os
from uuid import uuid4

from allauth.headless.base.response import APIResponse
from allauth.headless.mfa import response
from allauth.headless.mfa.views import ManageTOTPView
from allauth.mfa.adapter import DefaultMFAAdapter, get_adapter
from allauth.mfa.totp.internal.auth import get_totp_secret
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ValidationError
from django.http import HttpResponse, JsonResponse
from django.shortcuts import render
from django.utils.translation import gettext_lazy as _
from django.views import View
from django.views.decorators.csrf import csrf_exempt

from core.storages import TinymceS3Storage
from core.utils.files import sanitize_filename

logger = logging.getLogger(__name__)


def robots_txt(request):
    if settings.DEBUG:
        lines = [
            "User-agent: *",
            "Disallow: /",
        ]
    else:
        lines = [
            "User-agent: *",
            "Disallow: /admin/",
            "Disallow: /api/",
            "Disallow: /upload_image",
            "Disallow: /accounts/",
            "Disallow: /_allauth/",
            "Disallow: /rosetta/",
            "Disallow: /tinymce/",
        ]
    return HttpResponse("\n".join(lines), content_type="text/plain")


class HomeView(View):
    template_name = "home.html"

    def get(self, request):
        return render(request, self.template_name, {})


@csrf_exempt
def csp_report(request):
    if request.method == "POST":
        try:
            report = json.loads(request.body.decode("utf-8"))
            logger.warning(f"CSP Violation: {json.dumps(report, indent=2)}")
        except json.JSONDecodeError:
            logger.error("Failed to decode CSP report")
    return HttpResponse(status=204)


@csrf_exempt
@login_required
def upload_image(request):
    USE_AWS = os.getenv("USE_AWS", "False") == "True"

    user = request.user
    if not user.is_superuser:
        return JsonResponse(
            {"Error Message": "You are not authorized to upload images"}
        )

    if request.method != "POST":
        return JsonResponse({"Error Message": "Wrong request"})

    file_obj = request.FILES["file"]
    file_name_suffix = file_obj.name.split(".")[-1].lower()
    if file_name_suffix not in ["jpg", "png", "gif", "jpeg"]:
        return JsonResponse(
            {
                "Error Message": f"Wrong file suffix ({file_name_suffix}), supported are .jpg, .png, .gif, .jpeg"
            }
        )

    if USE_AWS:
        storage = TinymceS3Storage()
        sanitized_name = sanitize_filename(file_obj.name)
        image_path = storage.save(sanitized_name, file_obj)
        image_url = storage.url(image_path)
        return JsonResponse(
            {"message": "Image uploaded successfully", "location": image_url}
        )

    upload_dir = os.path.normpath(
        os.path.join(settings.MEDIA_ROOT, "uploads/tinymce")
    )
    if not os.path.exists(upload_dir):
        os.makedirs(upload_dir)

    sanitized_name = sanitize_filename(file_obj.name)
    file_path = os.path.join(upload_dir, sanitized_name)
    file_path = os.path.normpath(file_path)

    if not file_path.startswith(upload_dir):
        raise ValidationError(_("Invalid file path"))

    if os.path.exists(file_path):
        sanitized_name = str(uuid4()) + "." + file_name_suffix
        file_path = os.path.join(upload_dir, sanitized_name)

    with open(file_path, "wb+") as f:
        for chunk in file_obj.chunks():
            f.write(chunk)

        debug = os.getenv("DEBUG", "False") == "True"
        location = (
            f"{settings.API_BASE_URL}{settings.MEDIA_URL}uploads/tinymce/"
            f"{sanitized_name}"
            if debug
            else f"{settings.MEDIA_URL}uploads/tinymce/{sanitized_name}"
        )

        return JsonResponse(
            {
                "message": "Image uploaded successfully",
                "location": location,
            }
        )


class TOTPSvgNotFoundResponse(APIResponse):
    def __init__(self, request, secret, totp_url, totp_svg):
        super().__init__(
            request,
            meta={
                "secret": secret,
                "totp_url": totp_url,
                "totp_svg": totp_svg,
            },
            status=404,
        )


class ManageTOTPSvgView(ManageTOTPView):
    def get(self, request, *args, **kwargs):
        authenticator = self._get_authenticator()
        if not authenticator:
            adapter: DefaultMFAAdapter = get_adapter()
            secret = get_totp_secret(regenerate=True)
            totp_url: str = adapter.build_totp_url(request.user, secret)
            totp_svg = adapter.build_totp_svg(totp_url)
            return TOTPSvgNotFoundResponse(request, secret, totp_url, totp_svg)
        return response.TOTPResponse(request, authenticator)



# Generated by Django 4.2.9 on 2024-02-10 14:39
import uuid

import django.core.validators
import django.db.models.deletion
import django.db.models.manager
import parler.fields
import parler.models
import tinymce.models
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="BlogAuthor",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "updated_at",
                    models.DateTimeField(auto_now=True, verbose_name="Updated At"),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "website",
                    models.URLField(blank=True, null=True, verbose_name="Website"),
                ),
            ],
            options={
                "verbose_name": "Blog Author",
                "verbose_name_plural": "Blog Authors",
                "ordering": ["-created_at"],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="BlogAuthorTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "bio",
                    models.TextField(blank=True, null=True, verbose_name="Bio"),
                ),
            ],
            options={
                "verbose_name": "Blog Author Translation",
                "db_table": "blog_blogauthor_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="BlogCategory",
            fields=[
                (
                    "sort_order",
                    models.IntegerField(
                        db_index=True,
                        editable=False,
                        null=True,
                        verbose_name="Sort Order",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "updated_at",
                    models.DateTimeField(auto_now=True, verbose_name="Updated At"),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("slug", models.SlugField(unique=True)),
                (
                    "image",
                    models.ImageField(
                        blank=True,
                        null=True,
                        upload_to="uploads/blog/",
                        verbose_name="Image",
                    ),
                ),
                ("lft", models.PositiveIntegerField(editable=False)),
                ("rght", models.PositiveIntegerField(editable=False)),
                (
                    "tree_id",
                    models.PositiveIntegerField(db_index=True, editable=False),
                ),
                ("level", models.PositiveIntegerField(editable=False)),
            ],
            options={
                "verbose_name": "Blog Category",
                "verbose_name_plural": "Blog Categories",
                "ordering": ["sort_order"],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="BlogCategoryTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        blank=True,
                        max_length=50,
                        null=True,
                        verbose_name="Name",
                    ),
                ),
                (
                    "description",
                    models.TextField(blank=True, null=True, verbose_name="Description"),
                ),
            ],
            options={
                "verbose_name": "Blog Category Translation",
                "db_table": "blog_blogcategory_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="BlogComment",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "updated_at",
                    models.DateTimeField(auto_now=True, verbose_name="Updated At"),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "is_approved",
                    models.BooleanField(default=False, verbose_name="Is Approved"),
                ),
            ],
            options={
                "verbose_name": "Blog Comment",
                "verbose_name_plural": "Blog Comments",
                "ordering": ["-created_at"],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="BlogCommentTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "content",
                    models.TextField(
                        blank=True,
                        max_length=1000,
                        null=True,
                        validators=[django.core.validators.MinLengthValidator(1)],
                        verbose_name="Content",
                    ),
                ),
            ],
            options={
                "verbose_name": "Blog Comment Translation",
                "db_table": "blog_blogcomment_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="BlogPost",
            fields=[
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "updated_at",
                    models.DateTimeField(auto_now=True, verbose_name="Updated At"),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "published_at",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="Published At"
                    ),
                ),
                (
                    "is_published",
                    models.BooleanField(default=False, verbose_name="Is Published"),
                ),
                (
                    "seo_title",
                    models.CharField(
                        blank=True,
                        max_length=70,
                        null=True,
                        verbose_name="Seo Title",
                    ),
                ),
                (
                    "seo_description",
                    models.TextField(
                        blank=True,
                        max_length=300,
                        null=True,
                        verbose_name="Seo Description",
                    ),
                ),
                (
                    "seo_keywords",
                    models.CharField(
                        blank=True,
                        max_length=255,
                        null=True,
                        verbose_name="Seo Keywords",
                    ),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("slug", models.SlugField(max_length=255, unique=True)),
                (
                    "image",
                    models.ImageField(
                        blank=True,
                        null=True,
                        upload_to="uploads/blog/",
                        verbose_name="Image",
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("DRAFT", "Draft"),
                            ("PUBLISHED", "Published"),
                            ("ARCHIVED", "Archived"),
                        ],
                        default="DRAFT",
                        max_length=20,
                        verbose_name="Status",
                    ),
                ),
                (
                    "featured",
                    models.BooleanField(default=False, verbose_name="Featured"),
                ),
                (
                    "view_count",
                    models.IntegerField(default=0, verbose_name="View Count"),
                ),
                (
                    "author",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="blog_post_author",
                        to="blog.blogauthor",
                    ),
                ),
                (
                    "category",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="blog_post_category",
                        to="blog.blogcategory",
                    ),
                ),
            ],
            options={
                "verbose_name": "Blog Post",
                "verbose_name_plural": "Blog Posts",
                "ordering": ["-published_at"],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="BlogTag",
            fields=[
                (
                    "sort_order",
                    models.IntegerField(
                        db_index=True,
                        editable=False,
                        null=True,
                        verbose_name="Sort Order",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "updated_at",
                    models.DateTimeField(auto_now=True, verbose_name="Updated At"),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                (
                    "active",
                    models.BooleanField(default=True, verbose_name="Active"),
                ),
            ],
            options={
                "verbose_name": "Blog Tag",
                "verbose_name_plural": "Blog Tags",
                "ordering": ["sort_order"],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
            managers=[
                ("active_tags", django.db.models.manager.Manager()),
            ],
        ),
        migrations.CreateModel(
            name="BlogTagTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        blank=True,
                        max_length=50,
                        null=True,
                        validators=[django.core.validators.MinLengthValidator(1)],
                        verbose_name="Name",
                    ),
                ),
                (
                    "master",
                    parler.fields.TranslationsForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="blog.blogtag",
                    ),
                ),
            ],
            options={
                "verbose_name": "Blog Tag Translation",
                "db_table": "blog_blogtag_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="BlogPostTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        blank=True,
                        max_length=255,
                        null=True,
                        verbose_name="Title",
                    ),
                ),
                (
                    "subtitle",
                    models.CharField(
                        blank=True,
                        max_length=255,
                        null=True,
                        verbose_name="Subtitle",
                    ),
                ),
                (
                    "body",
                    tinymce.models.HTMLField(
                        blank=True, null=True, verbose_name="Body"
                    ),
                ),
                (
                    "master",
                    parler.fields.TranslationsForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="blog.blogpost",
                    ),
                ),
            ],
            options={
                "verbose_name": "Blog Post Translation",
                "db_table": "blog_blogpost_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-17 15:40
from decimal import Decimal

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("vat", "0003_vat_vat_vat_value_d54579_btree"),
    ]

    operations = [
        migrations.AlterField(
            model_name="vat",
            name="value",
            field=models.DecimalField(
                decimal_places=1,
                default=Decimal("0"),
                max_digits=11,
                verbose_name="Value",
            ),
        ),
    ]



from django.test import TestCase
from django.http import QueryDict
from django.db import models
from core.filters.camel_case_filters import (
    CamelCaseFilterMixin,
    CamelCaseTimeStampFilterSet,
    snake_to_camel,
)
from django_filters import rest_framework as filters


class MockModel(models.Model):
    name = models.CharField(max_length=255)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    sort_order = models.IntegerField(default=0)
    is_published = models.BooleanField(default=False)

    class Meta:
        app_label = "test_app"
        verbose_name = "Test Model"
        verbose_name_plural = "Test Models"

    def __str__(self):
        return self.name


class TestSnakeToCamel(TestCase):
    def test_basic_conversions(self):
        test_cases = [
            ("created_at", "createdAt"),
            ("updated_at", "updatedAt"),
            ("created_after", "createdAfter"),
            ("created_before", "createdBefore"),
            ("sort_order", "sortOrder"),
            ("is_published", "isPublished"),
            ("private_metadata", "privateMetadata"),
            ("country_name", "countryName"),
            ("alpha_exact", "alphaExact"),
            ("sort_order_min", "sortOrderMin"),
            ("metadata_has_key", "metadataHasKey"),
            ("currently_published", "currentlyPublished"),
        ]

        for snake_case, expected_camel in test_cases:
            with self.subTest(snake_case=snake_case):
                result = snake_to_camel(snake_case)
                self.assertEqual(result, expected_camel)

    def test_edge_cases(self):
        test_cases = [
            ("alpha", "alpha"),
            ("a", "a"),
            ("", ""),
            ("UPPERCASE", "UPPERCASE"),
            (
                "multiple_word_field_name",
                "multipleWordFieldName",
            ),
        ]

        for snake_case, expected_camel in test_cases:
            with self.subTest(snake_case=snake_case):
                result = snake_to_camel(snake_case)
                self.assertEqual(result, expected_camel)


class MockFilterSet(CamelCaseFilterMixin, filters.FilterSet):
    created_after = filters.DateTimeFilter(
        field_name="created_at",
        lookup_expr="gte",
    )
    sort_order = filters.NumberFilter(
        field_name="sort_order",
    )
    is_published = filters.BooleanFilter(
        field_name="is_published",
    )

    class Meta:
        model = MockModel
        fields = ["created_at", "sort_order", "is_published"]


class TestCamelCaseFilterMixin(TestCase):
    def test_filter_name_conversion(self):
        filterset = MockFilterSet()

        self.assertIn("createdAfter", filterset.base_filters)
        self.assertIn("sortOrder", filterset.base_filters)
        self.assertIn("isPublished", filterset.base_filters)

        self.assertNotIn("created_after", filterset.base_filters)
        self.assertNotIn("sort_order", filterset.base_filters)
        self.assertNotIn("is_published", filterset.base_filters)

    def test_field_name_preservation(self):
        filterset = MockFilterSet()

        self.assertEqual(
            filterset.base_filters["createdAfter"].field_name, "created_at"
        )
        self.assertEqual(
            filterset.base_filters["sortOrder"].field_name, "sort_order"
        )
        self.assertEqual(
            filterset.base_filters["isPublished"].field_name, "is_published"
        )

    def test_query_dict_conversion(self):
        query_dict = QueryDict(mutable=True)
        query_dict["createdAfter"] = "2024-01-01"
        query_dict["sortOrder"] = "1"
        query_dict["isPublished"] = "true"

        filterset = MockFilterSet(data=query_dict)

        self.assertIsNotNone(filterset.data)

    def test_regular_dict_conversion(self):
        data = {
            "createdAfter": "2024-01-01",
            "sortOrder": 1,
            "isPublished": True,
        }

        filterset = MockFilterSet(data=data)

        self.assertIsNotNone(filterset.data)

    def test_mixed_case_parameters(self):
        query_dict = QueryDict(mutable=True)
        query_dict["createdAfter"] = "2024-01-01"
        query_dict["sort_order"] = "1"

        filterset = MockFilterSet(data=query_dict)

        self.assertIsNotNone(filterset.data)

    def test_camel_case_fields_mapping(self):
        filterset = MockFilterSet()

        expected_mapping = {
            "createdAfter": "created_after",
            "createdAt": "created_at",
            "sortOrder": "sort_order",
            "isPublished": "is_published",
        }

        self.assertEqual(filterset._camel_case_fields, expected_mapping)


class TestCamelCaseTimeStampFilterSet(TestCase):
    def test_timestamp_filters_camelized(self):
        class TestFilterSet(CamelCaseTimeStampFilterSet):
            class Meta:
                model = MockModel
                fields = ["created_at", "updated_at"]

        filterset = TestFilterSet()

        self.assertIn("createdAfter", filterset.base_filters)
        self.assertIn("createdBefore", filterset.base_filters)
        self.assertIn("updatedAfter", filterset.base_filters)
        self.assertIn("updatedBefore", filterset.base_filters)

        self.assertEqual(
            filterset.base_filters["createdAfter"].field_name, "created_at"
        )
        self.assertEqual(
            filterset.base_filters["updatedBefore"].field_name, "updated_at"
        )



# Generated by Django 5.2.1 on 2025-05-23 14:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('product', '0022_alter_productfavourite_options_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='productimage',
            name='thumbnail',
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-18 12:03
from django.conf import settings
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("blog", "0011_alter_blogcommenttranslation_content_and_more"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AlterField(
            model_name="blogcomment",
            name="likes",
            field=models.ManyToManyField(
                blank=True,
                related_name="liked_comments",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="blogpost",
            name="likes",
            field=models.ManyToManyField(
                blank=True,
                related_name="liked_blog_posts",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



import os

from django.contrib.postgres.indexes import BTreeIndex
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_stubs_ext.db.models import TypedModelMeta
from mptt.fields import TreeForeignKey
from mptt.models import MPTTModel
from parler.models import TranslatableModel, TranslatedFields

from blog.managers.category import BlogCategoryManager
from blog.models.post import BlogPost
from core.fields.image import ImageAndSvgField
from core.models import SortableModel, TimeStampMixinModel, UUIDModel
from core.utils.generators import SlugifyConfig, unique_slugify


class BlogCategory(
    TranslatableModel, TimeStampMixinModel, SortableModel, UUIDModel, MPTTModel
):
    id = models.BigAutoField(primary_key=True)
    slug = models.SlugField(_("Slug"), max_length=255, unique=True)
    image = ImageAndSvgField(
        _("Image"), upload_to="uploads/blog/", blank=True, null=True
    )
    parent = TreeForeignKey(
        "self",
        blank=True,
        null=True,
        related_name="children",
        on_delete=models.CASCADE,
    )
    translations = TranslatedFields(
        name=models.CharField(_("Name"), max_length=50, blank=True, null=True),
        description=models.TextField(_("Description"), blank=True, null=True),
    )

    objects: BlogCategoryManager = BlogCategoryManager()

    class Meta(TypedModelMeta):
        verbose_name = _("Blog Category")
        verbose_name_plural = _("Blog Categories")
        ordering = ["sort_order"]
        indexes = [
            *TimeStampMixinModel.Meta.indexes,
            *SortableModel.Meta.indexes,
            BTreeIndex(fields=["slug"], name="blog_category_slug_ix"),
            BTreeIndex(fields=["parent"], name="blog_category_parent_ix"),
        ]

    class MPTTMeta:
        order_insertion_by = ["sort_order"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.sub_categories_list = None

    def __str__(self):
        if not hasattr(self, "_full_path"):
            self._full_path = " / ".join(
                [
                    k.safe_translation_getter(
                        "name", default="Unnamed", any_language=True
                    )
                    or "Unnamed"
                    for k in self.get_ancestors(include_self=True)
                ]
            )
        return self._full_path

    def save(self, *args, **kwargs):
        if not self.slug:
            config = SlugifyConfig(instance=self, title_field="name")
            self.slug = unique_slugify(config)
        super().save(*args, **kwargs)

    def get_ordering_queryset(self):
        return BlogCategory.objects.filter(parent=self.parent).get_descendants(
            include_self=True
        )

    @property
    def recursive_post_count(self) -> int:
        return BlogPost.objects.filter(
            category__in=self.get_descendants(include_self=True)
        ).count()

    @property
    def main_image_path(self) -> str:
        if self.image and hasattr(self.image, "name"):
            return f"media/uploads/blog/{os.path.basename(self.image.name)}"
        return ""

    @property
    def post_count(self):
        return self.blog_posts.count()



from .notification import NotificationFactory
from .notification import NotificationTranslationFactory
from .user import NotificationUserFactory

__all__ = [
    "NotificationFactory",
    "NotificationTranslationFactory",
    "NotificationUserFactory",
]



import binascii
import logging
from hmac import compare_digest
from typing import TYPE_CHECKING
from urllib.parse import parse_qs

from channels.db import database_sync_to_async
from channels.middleware import BaseMiddleware
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.utils import timezone
from knox.crypto import hash_token
from knox.settings import CONSTANTS, knox_settings

if TYPE_CHECKING:
    from knox.models import AuthToken

logger = logging.getLogger(__name__)

User = get_user_model()


@database_sync_to_async
def get_user(user_id: int):
    try:
        return User.objects.get(id=user_id)
    except User.DoesNotExist:
        return AnonymousUser()


@database_sync_to_async
def authenticate_token(token: str):
    from django.contrib.auth.models import AnonymousUser  # noqa: PLC0415
    from knox.models import get_token_model  # noqa: PLC0415

    logger.debug(
        f"Authenticating token: {token[:10] if token and isinstance(token, str) else 'None'}..."
    )

    try:
        token = token.strip()
    except AttributeError:
        logger.debug("Token has no strip method, returning AnonymousUser")
        return AnonymousUser()

    for auth_token in get_token_model().objects.filter(
        token_key=token[: CONSTANTS.TOKEN_KEY_LENGTH]
    ):
        logger.debug(
            f"Found token with key: {token[: CONSTANTS.TOKEN_KEY_LENGTH]}"
        )

        if _cleanup_token(auth_token):
            logger.debug("Token expired and was cleaned up")
            continue

        try:
            digest = hash_token(token)
        except (TypeError, binascii.Error) as e:
            logger.debug(f"Error hashing token: {e!s}")
            continue

        if compare_digest(digest, auth_token.digest):
            logger.debug("Token digest match")
            if knox_settings.AUTO_REFRESH and auth_token.expiry:
                logger.debug("Auto refreshing token")
                _renew_token(auth_token)

            user = (
                auth_token.user
                if auth_token.user.is_active
                else AnonymousUser()
            )
            logger.debug(
                f"Authenticated user: {user.username if not user.is_anonymous else 'AnonymousUser'}"
            )
            return user

        logger.debug("Token digest mismatch")

    logger.debug("No valid token found, returning AnonymousUser")
    return AnonymousUser()


def _renew_token(auth_token: "AuthToken"):
    current_expiry = auth_token.expiry
    new_expiry = timezone.now() + knox_settings.TOKEN_TTL
    auth_token.expiry = new_expiry
    delta = (new_expiry - current_expiry).total_seconds()
    if delta > knox_settings.MIN_REFRESH_INTERVAL:
        auth_token.save(update_fields=("expiry",))
        logger.debug(f"Token renewed, new expiry: {new_expiry}")


def _cleanup_token(auth_token: "AuthToken"):
    if auth_token.expiry is not None and auth_token.expiry < timezone.now():
        auth_token.delete()
        return True
    return False


class TokenAuthMiddleware(BaseMiddleware):
    async def __call__(self, scope, receive, send):
        logger.debug("TokenAuthMiddleware called")
        query_string = scope.get("query_string", b"").decode()
        query_params = parse_qs(query_string)

        logger.debug(f"Query params: {query_params}")

        session_token = query_params.get("session_token", [None])[0]
        access_token = query_params.get("access_token", [None])[0]
        user_id = query_params.get("user_id", [None])[0]

        logger.debug(
            f"Session token: {session_token[:10] if session_token else None}..."
        )
        logger.debug(
            f"Access token: {access_token[:10] if access_token else None}..."
        )
        logger.debug(f"User ID: {user_id}")

        token = access_token or session_token
        if token:
            logger.debug("TokenAuthMiddleware Token found, authenticating")
            user = await authenticate_token(token)
            scope["user"] = user
            logger.debug(
                f"User authenticated: {user.username if not user.is_anonymous else 'AnonymousUser'}"
            )
        elif user_id:
            logger.debug(f"No token found, but user_id is present: {user_id}")
            user = await get_user(int(user_id))
            scope["user"] = user
            logger.debug(
                f"User retrieved by ID: {user.username if not user.is_anonymous else 'AnonymousUser'}"
            )
        else:
            from django.contrib.auth.models import AnonymousUser  # noqa: PLC0415, I001

            logger.debug("TokenAuthMiddleware Token not found and no user_id")
            scope["user"] = AnonymousUser()

        return await super().__call__(scope, receive, send)


def TokenAuthMiddlewareStack(inner):
    return TokenAuthMiddleware(inner)
